-- ==================================================================
-- Author:		M. Groß, Raynet GmbH
-- Create date: 08.11.2019
-- Copyright:	(c) Raynet GmbH
-- Description: Cleanup duplicated devices with outdated inventories
-- ==================================================================

--Copy Network devices (rename ComputerID column to make join possible)
select DeviceID, DuplicateID, ComputerID as CID, DeviceName, DNSFullName, IPAddress, IPSubnet, IPSubnetMask, PhysicalAddress, OperatingSystem, FirstDiscovered, LastUpdate
  INTO #NetworkDevices
  FROM [ManageSoft].[dbo].[NetworkDevice];


--Remove devices with duplicate ComputerID
WITH cte AS (
  select *, ROW_NUMBER() OVER(
			PARTITION BY 
                CID
            ORDER BY 
                CID
  ) row_num
  from #NetworkDevices
)
DELETE FROM cte
WHERE row_num > 1;


--Join inventory timestamps to table (only for devices with duplicate)
	SELECT *
	INTO #Duplicates
	FROM #NetworkDevices as ND
	LEFT JOIN InventoryReport as IR
	ON ND.CID = IR.ComputerID
	where DuplicateID is not null or DeviceID in (select Duplicateid from #NetworkDevices)


--Copy table
SELECT *
INTO #DuplicatesCopy
FROM #Duplicates

--Collect devices with outdated inventories
SELECT DISTINCT n1.*
INTO #ComputersToDelete
FROM #Duplicates n1, #DuplicatesCopy n2
WHERE n1.DeviceID = n2.DuplicateID
	AND n1.SWDate < n2.SWDate

INSERT
INTO #ComputersToDelete SELECT n1.*
FROM #Duplicates n1, #DuplicatesCopy n2
WHERE n1.DuplicateID = n2.DeviceID
	AND n1.SWDate < n2.SWDate
	AND n1.DeviceID not in (SELECT DeviceID FROM #ComputersToDelete)

--Show duplicate devices with outdated inventories
SELECT *
FROM #ComputersToDelete
order by PhysicalAddress

--Show duplicate devices with newest inventories
SELECT *
FROM #Duplicates
where DeviceID not in ( SELECT DeviceID FROM #ComputersToDelete)

--Clean DuplicateID of devices to keep
SELECT *
INTO #DevicesToKeep
FROM #Duplicates
where DeviceID not in ( SELECT DeviceID FROM #ComputersToDelete)

UPDATE NetworkDevice
SET DuplicateID = NULL
WHERE DeviceID in (SELECT DeviceID FROM #DevicesToKeep)


--Delete devices with outdated inventories
DECLARE @DeviceIDToDelete INT

DECLARE curs CURSOR LOCAL FAST_FORWARD
	FOR SELECT DeviceID FROM #ComputersToDelete

OPEN curs

FETCH NEXT FROM curs INTO @DeviceIDToDelete

WHILE @@FETCH_STATUS = 0
	BEGIN
		EXEC RemoveNetworkDevice  @NetworkDeviceID = @DeviceIDToDelete, @ClearActionApplies = 1
		FETCH NEXT FROM curs INTO @DeviceIDToDelete
	END

CLOSE curs
DEALLOCATE curs

--Delete computers with outdated inventories
DECLARE @ComputerIDToDelete INT

DECLARE curs CURSOR LOCAL FAST_FORWARD
	FOR SELECT CID FROM #ComputersToDelete

OPEN curs

FETCH NEXT FROM curs INTO @ComputerIDToDelete

WHILE @@FETCH_STATUS = 0
	BEGIN
		EXEC DeleteComputer  @ComputerIDToDelete
		FETCH NEXT FROM curs INTO @ComputerIDToDelete
	END

CLOSE curs
DEALLOCATE curs

--Drop tables
DROP TABLE #NetworkDevices
DROP TABLE #Duplicates
DROP TABLE #DuplicatesCopy
DROP TABLE #ComputersToDelete
DROP TABLE #DevicesToKeep

